# -*- coding: utf-8 -*-
import subprocess

from scake import SckLog

from .base_action import BaseAction

sck_log = SckLog()


class ExeAction(BaseAction):
    def __init__(self, preset_manager, list_action, run_line_split):
        super().__init__(
            preset_manager=preset_manager,
            list_action=list_action,
            run_line_split=run_line_split,
        )

    def __call__(self, args=[]):
        command_line = self.list_action(args)
        if command_line:
            output = subprocess.getoutput("ls -l")
            print(self.run_line_split) if self.run_line_split else None
            print(output)
        else:  # print with `attis ls` then do nothing!
            pass


log_info = sck_log.register(obj_or_class=ExeAction, is_info=True)



# -*- coding: utf-8 -*-
from prettytable import PrettyTable
from scake import SckLog

from .base_action import BaseAction

sck_log = SckLog()


class HelpAction(BaseAction):
    def __init__(self, action_manifest):
        super().__init__(action_manifest=action_manifest)

    def __call__(self, args=[]):
        """
        action_manifest:
            help:
                alias: ["h"]
                description: Listing available actions and examples
                example:
                - a --help
                - a -h
                - a help
                - a h
            version:
                alias: ["v"]
                description: Version of installed Attis
                example:
                - a --version
                - a -v
                - a version
                - a v
        """
        print(
            """|----------------------------------------|
|                 ATTIS                  |
| A productivity tool for linux terminal |
|----------------------------------------|
Usage:  attis COMMAND [ARGUMENT]
or:     a COMMAND [ARGUMENT]
"""
        )
        help_table = PrettyTable()
        help_table.field_names = ["Command", "Description & Examples"]
        help_table.align = "l"
        help_table.border = False  # no_border
        col_max_width = 75
        help_table._max_width = {"Command": 30, "Description & Examples": col_max_width}
        help_table._min_width = {"Command": 30}
        for action_name, action_help in self.action_manifest.items():
            command = "\n".join(action_help.get("command", ""))
            description = action_help.get("description", "")
            examples = "\n".join(action_help.get("example", []))
            help_table.add_row(
                [
                    command,
                    "%s\n```bash\n%s\n```\n%s"
                    % (
                        description,
                        examples,
                        "-" * min(len(description), col_max_width),
                    ),
                ]
            )
        print(help_table)


log_info = sck_log.register(obj_or_class=HelpAction, is_info=True)



# -*- coding: utf-8 -*-
import os

from omegaconf import OmegaConf
from scake import SckLog

sck_log = SckLog()

BOOK_PAGE_SPLIT_DELIMITER = "/"

SOURCE_KEY_TYPE = "type"
SOURCE_KEY_ENDPOINT = "endpoint"
SOURCE_KEY_BOOK = "book"

SOURCE_TYPE_LOCAL = "local"
SOURCE_TYPE_REMOTE = "remote"


class PresetManager:
    # @plazy.auto_assign
    def __init__(self, sources, local_preset_name="local.yml"):
        self.sources = sources
        self.local_preset_name = local_preset_name
        self.book = OmegaConf.create()
        self.source_dict = self.load(self.sources)

    def load(self, sources):
        """
        sources: ['/etc/attis/preset/']
        """
        result = {}
        sources = (
            [
                sources,
            ]
            if not isinstance(sources, (list, tuple))
            else sources
        )
        for src in list(sources):
            result.setdefault(src, {})
            if src.startswith("http") or src.startswith("https"):
                result[src].update(
                    {
                        SOURCE_KEY_TYPE: SOURCE_TYPE_REMOTE,
                        SOURCE_KEY_ENDPOINT: False,  # TODO
                        SOURCE_KEY_BOOK: OmegaConf.create(),  # TODO
                    }
                )
                continue
            else:  # local folder
                os.makedirs(src, exist_ok=True)
                local_endpoint_file = os.path.join(
                    src, self.local_preset_name
                )  # /etc/attis/preset/local.yml
                if not os.path.isfile(local_endpoint_file):
                    with open(local_endpoint_file, "w"):
                        pass
                src_book = OmegaConf.load(local_endpoint_file)
                self.book = OmegaConf.merge(self.book, src_book)
                result[src].update(
                    {
                        SOURCE_KEY_TYPE: SOURCE_TYPE_LOCAL,
                        SOURCE_KEY_ENDPOINT: local_endpoint_file,  # TODO
                        SOURCE_KEY_BOOK: src_book,
                    }
                )
        return result

    def set_all_book(self, key, value):
        for src in self.source_dict.keys():
            self.set_book(src=src, key=key, value=value)

    def set_book(self, src, key, value):
        book_type = self.source_dict.get(src, {}).get(
            SOURCE_KEY_TYPE, False
        )  # local file path or remote url
        book_endpoint = self.source_dict.get(src, {}).get(
            SOURCE_KEY_ENDPOINT, False
        )  # local file path or remote url
        if not book_endpoint:
            print(
                "Cannot set %s book (%s) at endpoint: %s"
                % (book_type, src, book_endpoint)
            )
        if book_type not in (SOURCE_TYPE_LOCAL, SOURCE_TYPE_REMOTE):
            print("Unsupported book type (%s)" % book_type)

        if book_type == SOURCE_TYPE_LOCAL:
            dot_list = [
                '%(key)s="%(value)s"'
                % {
                    "key": key.replace(BOOK_PAGE_SPLIT_DELIMITER, "."),
                    "value": value,
                }
            ]
            new_book = OmegaConf.from_dotlist(dot_list)
            self.book = OmegaConf.merge(self.book, new_book)
            self.source_dict[src][SOURCE_KEY_BOOK] = OmegaConf.merge(
                self.source_dict[src][SOURCE_KEY_BOOK], new_book
            )
        else:
            print("TODO")
            raise Exception()

        self.save_source(src)

    def save_source(self, src):
        src_type = self.source_dict[src][SOURCE_KEY_TYPE]
        src_endpoint = self.source_dict[src][SOURCE_KEY_ENDPOINT]
        if src_type == SOURCE_TYPE_LOCAL:  # local file => dump to yml file
            src_book = self.source_dict[src][SOURCE_KEY_BOOK]
            with open(src_endpoint, "w") as f:
                f.write(OmegaConf.to_yaml(src_book))

    def get(self, target=False, key=False, default=False, throw_on_missing=False):
        target = self.book if not target else target
        if not key:
            return target
        else:
            return OmegaConf.select(
                target,
                key.replace(BOOK_PAGE_SPLIT_DELIMITER, "."),
                default=default,
                throw_on_missing=False,
            )


log_info = sck_log.register(obj_or_class=PresetManager, is_info=True)



# -*- coding: utf-8 -*-
import plazy
from scake import SckLog

sck_log = SckLog()


class BaseAction:
    @plazy.auto_assign
    def __init__(self):
        pass

    def __call__(self, args=[]):
        log_error("Not implemented method!")
        raise Exception()


log_error = sck_log.register(obj_or_class=BaseAction, is_error=True)



# -*- coding: utf-8 -*-
import plazy
from scake import SckLog

sck_log = SckLog()


class ActionManager:
    @plazy.auto_assign
    def __init__(self, actions={}, preset_manager=False):
        pass

    def __call__(self, args=[]):
        p_args = []
        for a in args:
            if not a:
                continue
            item = a.strip()
            p_args.append(item)

        if p_args and len(p_args) > 0:
            action_name = p_args[0]
            action_args = p_args[1:]

            action_name = (
                action_name[2:] if action_name.startswith("--") else action_name
            )
            action_name = (
                action_name[1:] if action_name.startswith("-") else action_name
            )

            action_object = self.actions.get(action_name, False)
            if action_object is not False:
                action_object(
                    action_args.copy()
                )  # passing arguments to action resolver
            else:  # no action_name matched!
                run_action = self.actions.get("run", False)
                if run_action is not False:
                    run_action(p_args.copy())
                else:
                    log_error("Attis Internal Error: Run Action not found")
                    raise Exception()
        else:  # attis "without any params"
            if self.preset_manager is False or self.preset_manager is None:
                log_error("Attis Internal Error: Preset Manager not found")
                raise Exception()

            if (
                self.preset_manager.book and len(self.preset_manager.book) == 0
            ) or not self.preset_manager.book:
                # no data -> show help
                help_action = self.actions["help"]
                help_action()
            else:
                ls_action = self.actions["ls"]
                ls_action()

    def test_main_entry(self, multiple_commands):
        multiple_commands = (
            [
                multiple_commands,
            ]
            if not isinstance(multiple_commands, (tuple, list))
            else multiple_commands
        )
        for cmd_args in multiple_commands:
            self.__call__(args=cmd_args.split(" "))
            print("-------------")


log_info = sck_log.register(obj_or_class=ActionManager, is_info=True)
log_error = sck_log.register(obj_or_class=ActionManager, is_error=True)



# -*- coding: utf-8 -*-
from omegaconf import OmegaConf
from prettytable import PrettyTable
from scake import SckLog

from ..preset_manager import BOOK_PAGE_SPLIT_DELIMITER
from .base_action import BaseAction

sck_log = SckLog()


class ListAction(BaseAction):
    def __init__(self, preset_manager):
        super().__init__(preset_manager=preset_manager)

    def get_pretty_table(self):
        res_table = PrettyTable()
        res_table.field_names = ["Name", "Cmd", "Path"]
        res_table.align = "l"
        res_table.border = True  # False  #no_border
        res_table.header = True  # False  #no_header
        return res_table

    def resolve_query_result(self, res, path=False):
        is_leaf = True
        if OmegaConf.is_config(res):
            is_leaf = False
            table = self.get_pretty_table()
            res = OmegaConf.to_object(res)  # dict or list
            if isinstance(res, dict):
                item_arr = []
                for k, v in res.items():
                    if isinstance(v, (dict, list, tuple)):
                        if len(v) == 0:
                            continue
                        item_arr.append(
                            (
                                "%s (%d)" % (k, len(v)),
                                "",
                                len(v),
                                BOOK_PAGE_SPLIT_DELIMITER.join([path, k])
                                if path
                                else k,
                            )
                        )  # key, value, weight, full_path
                    else:  # scalar
                        item_arr.append(
                            (
                                k,
                                v,
                                -1,
                                BOOK_PAGE_SPLIT_DELIMITER.join([path, k])
                                if path
                                else k,
                            )
                        )
                item_arr = sorted(item_arr, key=lambda tup: (tup[2], tup[0]))
                table.add_rows([[item[0], item[1], item[3]] for item in item_arr])
                res = table.get_string()
            # else: # list
            #     table.add_rows([[item, False] for item in res])
            #     res = table.get_string()
            # pass
        return res, is_leaf

    def __call__(self, args=[]):
        p_keys = []
        for arg in args:  # my_page/my_key
            if arg:
                p_keys += arg.split(BOOK_PAGE_SPLIT_DELIMITER)

        # first, try direct key query my_page1/my_page2/my_key
        clone_keys = p_keys.copy()
        while len(clone_keys) >= 0:
            if len(clone_keys) == 0:
                res, is_leaf = self.resolve_query_result(
                    self.preset_manager.get(target=False, key=False)
                )  # self.preset_manager.book
                print(res)
                return False

            direct_key = ".".join(clone_keys)
            res = self.preset_manager.get(key=direct_key, default=False)
            if res:
                cmd, is_leaf = self.resolve_query_result(
                    res, path=BOOK_PAGE_SPLIT_DELIMITER.join(clone_keys)
                )
                print(cmd)
                return cmd if is_leaf else False
            else:
                attis_key = BOOK_PAGE_SPLIT_DELIMITER.join(clone_keys)
                clone_keys = clone_keys[:-1]
                new_attis_key = BOOK_PAGE_SPLIT_DELIMITER.join(clone_keys)
                print(
                    "Attis cannot access '%s'. Try again with '%s'"
                    % (attis_key, new_attis_key)
                )

        # #TODO: ls with regex pattern, similar matching
        # res = self.preset_manager.get(target=False, key=False) # self.preset_manager.book
        # for idx, key in enumerate(p_keys):
        #     is_last_key = idx == len(p_keys)-1
        #     t_res = self.preset_manager.get(target=res, key=key)
        #     if t_res: # OK
        #         res = t_res
        #     else: # result not found, do advanced search
        #         pass
        # print(self.resolve_query_result(res))
        # pass


log_info = sck_log.register(obj_or_class=ListAction, is_info=True)



# -*- coding: utf-8 -*-
import os
import re

from scake import SckLog

from .base_action import BaseAction

sck_log = SckLog()


attis_init_file_path = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))),
    "__init__.py",
)


def get_property(file_python_path, prop):
    result = re.search(
        r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(file_python_path).read()
    )
    return result.group(1)


class VersionAction(BaseAction):
    def __init__(self):
        super().__init__()

    def __call__(self, args=[]):
        print(get_property(attis_init_file_path, "__version__"))


log_info = sck_log.register(obj_or_class=VersionAction, is_info=True)



# -*- coding: utf-8 -*-
from scake import SckLog

from .base_action import BaseAction

sck_log = SckLog()


class SetAction(BaseAction):
    def __init__(self, preset_manager):
        super().__init__(preset_manager=preset_manager)

    def __call__(self, args=[]):
        key = args[0]
        value = " ".join(args[1:])
        self.preset_manager.set_all_book(key=key, value=value)
        print("OK")


log_info = sck_log.register(obj_or_class=SetAction, is_info=True)



# -*- coding: utf-8 -*-
import logging
import os
import sys

from scake import Scake, SckLog

sck_log = SckLog()


logging.addLevelName(
    logging.INFO, "\x1b[1;34m%s\x1b[0m" % logging.getLevelName(logging.INFO)
)
logging.addLevelName(
    logging.WARNING, "\x1b[1;33m%s\033[1;0m" % logging.getLevelName(logging.WARNING)
)
logging.addLevelName(
    logging.ERROR, "\033[1;31m%s\033[1;0m" % logging.getLevelName(logging.ERROR)
)
_logger = logging.getLogger(__name__)

ATTIS_CONFIG_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "config.yml"
)


def run():
    pass


def main_entry():
    # print("------------------")
    # print("|     Attis      |")
    # print("------------------")

    sck = Scake(ATTIS_CONFIG_PATH)

    # setup logger
    if sck.get("/config/attis/mode", "prod") != "debug":
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)
        logging.basicConfig(
            stream=sys.stderr,
            level=logging.ERROR,
            format="%(asctime)s.%(msecs)03d %(levelname)s %(funcName)s: %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
    else:
        pass

    sck()


if __name__ == "__main__":
    main_entry()
